// --- [ at45db081.h ] --------------------------------------------------------

#define  PAGES_ON_CHIP		4096			// Anzahl der Pages auf dem Chip
#define  PAGE_MASK				PAGES_ON_CHIP - 1 // Maske fr Counterberlauf

#define  PAGE_SIZE		256
#define  STATUS_READY	164		// Bit.7 + Chipspezifische Bits = codierte Speichergre

											// Index/Bedeutung der Bytes im 'Header' jeder Page
#define  FILE_ID_HIGH		0		// laufende Nr./ID einer Datei, beginnend mit 1
#define  FILE_ID_LOW			1
#define	PAGE_SIZE_HIGH		2		// Anzahl der Bytes in der Page (nur Lower Nibble !)
											// Bit.16-20 der Zeit im Upper Nibble
#define	PAGE_SIZE_LOW		3		// Anzahl der benutzten Bytes in der jeweiligen Page
#define  PAGE_CNT_HIGH		4		// Fortlaufende Zhlung der Pages einer Datei
#define  PAGE_CNT_LOW		5
#define  PAGE_TIME_HIGH		6		// Zeit seit dem Start der LogDatei in Sekunden
#define  PAGE_TIME_LOW		7		// plus 4 Bit im Upper Nibble von P_SIZE_HIGH


// Table 15-1 -----------------------------------------------------------------

// main memory page read
#define MAIN_MEMORY_PAGE_READ 0xD2

// continuous Array Read (low frequency) 
#define CONT_READ_LOW 0x03

// continuous Array Read (high frequency) 
#define CONT_READ_HIGH 0x0B

// buffer 1 read low
#define BUFFER_1_READ_LOW 0xD1

// buffer 2 read low
#define BUFFER_2_READ_LOW 0xD3

// buffer 1 read
#define BUFFER_1_READ 0xD4

// buffer 2 read
#define BUFFER_2_READ 0xD6


// Table 15-2 -----------------------------------------------------------------

// buffer 1 write
#define BUFFER_1_WRITE	0x84

// buffer 2 write
#define BUFFER_2_WRITE	0x87

// buffer 1 to main memory page program with built-in erase
#define B1_TO_MM_PAGE_PROG_WITH_ERASE 0x83

// buffer 2 to main memory page program with built-in erase
#define B2_TO_MM_PAGE_PROG_WITH_ERASE 0x86

// buffer 1 to main memory page program without built-in erase
#define B1_TO_MM_PAGE_PROG_WITHOUT_ERASE 0x88

// buffer 2 to main memory page program without built-in erase
#define B2_TO_MM_PAGE_PROG_WITHOUT_ERASE 0x89

// erase a 528 byte page
#define PAGE_ERASE 0x81

// erase 512 pages
#define BLOCK_ERASE 0x50

// erase sector
#define SECTOR_ERASE 0x7c

// main memory page program through buffer 1
#define MM_PAGE_PROG_THROUGH_B1 0x82
 
// main memory page program through buffer 2
#define MM_PAGE_PROG_THROUGH_B2 0x85


// Table 15-3 -----------------------------------------------------------------


 
 
 
// Table 15-4 -----------------------------------------------------------------

// main memory page to buffer 1 transfer
#define MM_PAGE_TO_B1_XFER 0x53

// main memory page to buffer 2 transfer
#define MM_PAGE_TO_B2_XFER 0x55

// main memory page compare to buffer 1
#define MM_PAGE_TO_B1_COMP 0x60

// main memory page compare to buffer 2
#define MM_PAGE_TO_B2_COMP 0x61
 
// auto page rewrite through buffer 1
#define AUTO_PAGE_REWRITE_THROUGH_B1 0x58
 
// auto page rewrite through buffer 2
#define AUTO_PAGE_REWRITE_THROUGH_B2 0x59
 
// deep power-down
#define POWER_DOWN	0xB9

// resume from power-down
#define RESUME_POWER_DOWN	0xAB

// status register
#define STATUS_REGISTER	0xD7

// device id
#define DEVICE_ID 0x9F

void DF_write_page(uint8_t idx);
void DF_erase_page(uint16_t page);
uint8_t DF_device_ready(void);
void DF_read_device_id(void);
uint8_t DF_compare_failed(void);
void DF_check(void);
void DF_read_byte(uint16_t page, uint16_t databyte, uint8_t cnt);
void DF_read_page(uint16_t page, uint16_t byte_adr);
void DF_erase_chip(void);
uint16_t DF_print_file(uint16_t lastpage);
void DF_print_page(uint16_t page);
void DF_file_test(void);
// --- [ eof ] ----------------------------------------------------------------

 

